
class ObjectNotFound(Exception):
    ...


class ObjectNotSupported(Exception):
    ...


class Forbidden(Exception):
    ...


class Unauthorized(Exception):
    ...


class UnknownError(Exception):
    ...


class MindNameInvalid(Exception):
    ...


class DatasourceNameInvalid(Exception):
    ...