__title__ = 'minds_sdk'
__package_name__ = 'minds'
__version__ = "2.0.1"
__description__ = 'An AI-Data Mind is an LLM with the built-in power to answer data questions for Agents'
__email__ = 'hello@mindsdb.com'
__author__ = 'MindsDB Inc'
__github__ = 'https://github.com/mindsdb/minds_python_sdk'
__pypi__ = 'https://pypi.org/project/mindsdb/'
__copyright__ = 'Copyright 2024-MindsDB'
